// Copyright (C) Menhirs NV. All rights reserved.

#include "StdAfx.h"


#ifdef _WIN32 // only under Windows
#endif // _WIN32


//SR

#if BRX_APP
#include <afxwin.h>
#include "acgi.h"
#include <BrxSpecific/AcSharedPtr.h>
#include "BrxSpecific/BcUiPanelMFC.h"
#include "BrxSpecific/BcUiPanel.h"
#include <QWidget>
#include "AdAChar.h"
#include "qwindow.h"

#include <QGuiApplication>
#include <QWidget>
#include <QLineEdit>
#include <QComboBox>
#include <QVBoxLayout>
#include <QPushButton>

class MyPanel : public BcUiPanelMFC
{
public:
    MyPanel();
    ~MyPanel();
    void setWindow();
    QWidget* widget;

protected:
    virtual BOOL CreateControlBar(LPCREATESTRUCT lpCreateStruct) override;
};

static CButton mButton;

MyPanel::MyPanel() : BcUiPanelMFC(ACRX_T("My BcUiPanelMFC panel with Qt embedded controls"), ACRX_T("BcUiPanelMFC_Qt"))
{
    RegisterRestartableTool(ACRX_T("_TMY")); //Owen
    widget = new QWidget;
    QVBoxLayout* vLayout = new QVBoxLayout();
    QLineEdit* lineEdit = new QLineEdit;
    QComboBox* comboBox = new QComboBox;
    QPushButton* pushButton = new QPushButton("Hello World", widget);
    pushButton->setGeometry(QRect(QPoint(0, 20), QSize(100, 55)));
    comboBox->addItems({ "item 1", "item 2", "item 3", "item 4" });
    vLayout->addWidget(lineEdit);
    vLayout->addWidget(comboBox);
    vLayout->setContentsMargins(110, 20, 20, 20);
    widget->setLayout(vLayout);
}

MyPanel::~MyPanel()
{
}

BOOL MyPanel::CreateControlBar(LPCREATESTRUCT lpCreateStruct)
{
    if (!BcUiPanelMFC::CreateControlBar(lpCreateStruct))
    {
        return FALSE;
    }
    CRect rcClient;
    GetClientRect(&rcClient);
    setWindow();
    return TRUE;
}

void MyPanel::setWindow() {
    WId wid = WId(m_hWnd);
    QWindow* pQwin = QWindow::fromWinId(wid);
    pQwin->setProperty("Qt_embedded_native_parent_handle", QVariant::fromValue(wid));
    widget->winId();
    widget->windowHandle()->setParent(pQwin);
    widget->move(0, 0);
    widget->show();
    SetFocus();
}


//test
void initApp();
void unloadApp();
void addMyPanel() {
    static MyPanel panel;
    static bool init = false;


    if (panel.m_hWnd)
    {

    }
    else if (!init)
    {
        panel.Create();
        init = true;

        acutPrintf(_T("\nThe native style panel sample \"My Custom Panel\" is now included in the panel menu and can be shown, docked, etc. like native panels."));// eug

        //known, reported issues:
        acutPrintf(_T("\n => Issue 1: pick the QComboBox: the combo sub - items window is placed below with an offset / distance, instead of adjacent to the combo"));

        acutPrintf(_T("\n => Issue 2: type a few characters in QLineEdit: they are wrongly moved in commandline instead of that apparently focused QLineEdit."));
    }
}

void initApp() {
    acedRegCmds->addCommand(
        _T("mypalette"), _T("addMyPanel"), _T("SR157985_addMyPanel"),
        ACRX_CMD_MODAL | ACRX_CMD_USEPICKSET | ACRX_CMD_REDRAW,
        addMyPanel);

}

void unloadApp() {
    acedRegCmds->removeGroup(_T("mypalette"));
}

#else ARX_APP

#endif
void MySandboxCommand(void)
{
    acutPrintf(_T("\nThe panel with Qt controls is initialized. Now running 'SR157985_addMyPanel' command..."));
#if BRX_APP
    acedCommand(
        RTSTR, _T("SR157985_addMyPanel"),
        RTNONE);
#else ARX_APP

#endif
}

class CSampleApp : public AcRxArxApp
{
public:
    CSampleApp() : AcRxArxApp() {}

    virtual void RegisterServerComponents()
    {
    }

    virtual AcRx::AppRetCode On_kInitAppMsg(void* pAppData)
    {
        AcRx::AppRetCode result = AcRxArxApp::On_kInitAppMsg(pAppData);

        acrxRegisterAppMDIAware(pAppData); // is able to work in MDI context
        acrxUnlockApplication(pAppData);   // allows to unload the module during session

        //Place your init code here
        acutPrintf(_T("\n* Available commands:"));
#ifdef _WIN32 // only under Windows

        acutPrintf(_T("\n* TMY   : "));
        acutPrintf(_T("\n => Build Release (V22 or V23), having e.g.:\n$(QT51511_SDK_PATH)=d:\\Qt\\5.15.11\\msvc2019_64 (path needs an update)"));
#if (__BRXTARGET >= 21)

#endif // (__BRXTARGET >= 21)
#endif // _WIN32
        acutPrintf(_T("\n"));

#if BRX_APP
        //SR
        initApp();
#else ARX_APP

#endif

        return result;
    }

    virtual AcRx::AppRetCode On_kUnloadAppMsg(void* pAppData)
    {
        //Place your cleanup code here
#if BRX_APP
        //SR
        unloadApp();
#else ARX_APP

#endif

        return AcRxArxApp::On_kUnloadAppMsg(pAppData);
    }

    virtual AcRx::AppRetCode On_kLoadDwgMsg(void* pAppData)
    {
        return AcRxArxApp::On_kLoadDwgMsg(pAppData);
    }

    virtual AcRx::AppRetCode On_kUnloadDwgMsg(void* pAppData)
    {
        return AcRxArxApp::On_kUnloadDwgMsg(pAppData);
    }

    virtual AcRx::AppRetCode On_kQuitMsg(void* pAppData)
    {
        return AcRxArxApp::On_kQuitMsg(pAppData);
    }

#ifdef _WIN32 // only under Windows


    static void BrxSampleTMY(void)
    {
        MySandboxCommand();
    }

#endif // _WIN32
};

IMPLEMENT_ARX_ENTRYPOINT(CSampleApp)


#ifdef _WIN32 // only under Windows
ACED_ARXCOMMAND_ENTRY_AUTO(CSampleApp, BrxSample, TMY, TMY, ACRX_CMD_TRANSPARENT, NULL)
#endif // _WIN32
